import {
    ChangeDetectionStrategy,
    ChangeDetectorRef,
    Component,
    EventEmitter,
    inject,
    Input,
    OnDestroy,
    OnInit,
    Output,
    ViewChild
} from '@angular/core';
import { EnableOrDisableFlapDetectionItem, ExternalCommandsService } from '../../../services/external-commands.service';
import {
    ButtonCloseDirective,
    CardBodyComponent,
    CardComponent,
    ColComponent,
    ModalBodyComponent,
    ModalComponent,
    ModalFooterComponent,
    ModalHeaderComponent,
    ModalService,
    ModalTitleDirective,
    ProgressComponent,
    RowComponent
} from '@coreui/angular';
import { NotyService } from '../../../layouts/coreui/noty.service';
import { Subscription } from 'rxjs';
import { FaIconComponent } from '@fortawesome/angular-fontawesome';
import { TranslocoDirective, TranslocoService } from '@jsverse/transloco';
import { XsButtonDirective } from '../../../layouts/coreui/xsbutton-directive/xsbutton.directive';
import { HttpErrorResponse } from '@angular/common/http';


@Component({
    selector: 'oitc-hosts-disable-flapdetection-modal',
    imports: [
        ButtonCloseDirective,
        CardBodyComponent,
        CardComponent,
        ColComponent,
        FaIconComponent,
        ModalBodyComponent,
        ModalComponent,
        ModalFooterComponent,
        ModalHeaderComponent,
        ModalTitleDirective,
        RowComponent,
        TranslocoDirective,
        XsButtonDirective,
        ProgressComponent
    ],
    templateUrl: './hosts-disable-flapdetection-modal.component.html',
    styleUrl: './hosts-disable-flapdetection-modal.component.css',
    changeDetection: ChangeDetectionStrategy.OnPush
})
export class HostsDisableFlapdetectionModalComponent implements OnInit, OnDestroy {
    @Input({required: true}) public items: EnableOrDisableFlapDetectionItem[] = [];
    @Output() completed: EventEmitter<boolean> = new EventEmitter<boolean>();
    public hasErrors: boolean = false;

    private readonly ExternalCommandsService: ExternalCommandsService = inject(ExternalCommandsService);
    private readonly modalService: ModalService = inject(ModalService);
    private readonly notyService: NotyService = inject(NotyService);
    private readonly subscriptions: Subscription = new Subscription();
    private readonly TranslocoService = inject(TranslocoService);
    private cdr = inject(ChangeDetectorRef);

    public percentage: number = 0;
    public isSending: boolean = false;

    @ViewChild('modal') private modal!: ModalComponent;

    public hideModal() {
        this.isSending = false;
        this.percentage = 0;
        this.hasErrors = false;

        this.cdr.markForCheck();

        this.modalService.toggle({
            show: false,
            id: 'hostDisableFlapdetectionModal'
        });
    }

    public setExternalCommands() {
        this.isSending = true;
        this.sendCommands();
    }


    public ngOnInit() {
    }

    public ngOnDestroy() {
        this.subscriptions.unsubscribe();
    }

    public sendCommands() {

        this.percentage = 0;
        this.isSending = true;
        this.cdr.markForCheck();
        let count = this.items.length;
        let responseCount: number = 0
        let issueCount: number = 0;

        for (let i in this.items) {
            const item = this.items[i];

            this.ExternalCommandsService.flap_detection(item).subscribe({
                next: (value: any) => {
                    responseCount++
                    this.percentage = Math.round((responseCount / count) * 100);
                    this.cdr.markForCheck();

                    if (responseCount === count && issueCount === 0) {
                        // The timeout is not necessary. It is just to show the progress bar at 100% for a short time.
                        setTimeout(() => {
                            // All records have been sent successfully. Reset the modal
                            this.isSending = false;
                            this.percentage = 0;
                            this.hasErrors = false;

                            this.hideModal();
                        }, 250);

                        const title: string = this.TranslocoService.translate('Flap detection disabled');
                        const msg: string = this.TranslocoService.translate('Commands added successfully to queue');

                        this.notyService.genericSuccess(msg, title);

                        setTimeout(() => {
                            this.completed.emit(true);
                        }, 5000);
                    }

                },
                error: (error: HttpErrorResponse) => {
                    this.cdr.markForCheck();
                    issueCount++
                    this.hasErrors = true;

                    responseCount++;
                    this.percentage = Math.round((responseCount / count) * 100);

                    if (responseCount === count && issueCount > 0) {
                        // The timeout is not necessary. It is just to show the progress bar at 100% for a short time.
                        setTimeout(() => {
                            this.isSending = false;
                            this.percentage = 0;
                        }, 250);

                        this.notyService.genericError();

                        this.completed.emit(false);
                    }
                }
            });
        }

    }
}
