import {
    ChangeDetectionStrategy,
    ChangeDetectorRef,
    Component,
    EventEmitter,
    inject,
    Input,
    OnDestroy,
    OnInit,
    Output,
    ViewChild
} from '@angular/core';
import {
    ButtonCloseDirective,
    ColComponent,
    FormControlDirective,
    FormLabelDirective,
    ModalBodyComponent,
    ModalComponent,
    ModalFooterComponent,
    ModalHeaderComponent,
    ModalService,
    ModalTitleDirective,
    ProgressComponent,
    RowComponent
} from '@coreui/angular';
import { FaIconComponent } from '@fortawesome/angular-fontawesome';
import { FormsModule } from '@angular/forms';
import { Subscription } from 'rxjs';
import { TranslocoDirective, TranslocoService } from '@jsverse/transloco';
import { NotyService } from '../../../layouts/coreui/noty.service';
import { DowntimesDefaultsService, ValidateInput } from '../../../services/downtimes-defaults.service';
import { ExternalCommandsService, ServiceDowntimeItem } from '../../../services/external-commands.service';
import { XsButtonDirective } from '../../../layouts/coreui/xsbutton-directive/xsbutton.directive';
import { GenericValidationError } from '../../../generic-responses';
import { HttpErrorResponse } from '@angular/common/http';
import { FormErrorDirective } from '../../../layouts/coreui/form-error.directive';
import { FormFeedbackComponent } from '../../../layouts/coreui/form-feedback/form-feedback.component';


@Component({
    selector: 'oitc-service-maintenance-modal',
    imports: [
        ButtonCloseDirective,
        ColComponent,
        FaIconComponent,
        ModalBodyComponent,
        ModalComponent,
        ModalFooterComponent,
        ModalHeaderComponent,
        ModalTitleDirective,
        RowComponent,
        TranslocoDirective,
        FormControlDirective,
        FormLabelDirective,
        FormsModule,
        XsButtonDirective,
        FormErrorDirective,
        FormFeedbackComponent,
        ProgressComponent
    ],
    templateUrl: './service-maintenance-modal.component.html',
    styleUrl: './service-maintenance-modal.component.css',
    changeDetection: ChangeDetectionStrategy.OnPush
})
export class ServiceMaintenanceModalComponent implements OnInit, OnDestroy {
    @Input({required: true}) public items: ServiceDowntimeItem[] = [];
    @Input({required: false}) public maintenanceMessage: string = '';
    @Input({required: false}) public helpMessage: string = '';
    @Output() completed = new EventEmitter<boolean>();
    public hasErrors: boolean = false;
    public errors: GenericValidationError | null = null;
    public error: boolean = false;
    public state?: any
    public downtimeModal = {
        comment: '',
        from_date: '',
        from_time: '',
        to_date: '',
        to_time: ''
    };

    private readonly TranslocoService = inject(TranslocoService);
    private readonly DowntimesDefaultsService = inject(DowntimesDefaultsService);
    private readonly ExternalCommandsService = inject(ExternalCommandsService);
    private readonly modalService = inject(ModalService);
    private readonly notyService = inject(NotyService);

    private cdr = inject(ChangeDetectorRef);

    public percentage: number = 0;
    public isSending: boolean = false;

    private subscriptions: Subscription = new Subscription();
    @ViewChild('modal') private modal!: ModalComponent;

    hideModal() {
        this.isSending = false;
        this.hasErrors = false;
        this.errors = null;

        this.cdr.markForCheck();

        this.modalService.toggle({
            show: false,
            id: 'serviceMaintenanceModal'
        });
    }

    setExternalCommands() {
        this.error = false;
        this.isSending = true;
        const validate: ValidateInput = {
            comment: this.downtimeModal.comment,
            from_date: this.downtimeModal.from_date,
            from_time: this.downtimeModal.from_time,
            to_date: this.downtimeModal.to_date,
            to_time: this.downtimeModal.to_time
        }
        this.isSending = true;
        this.subscriptions.add(this.DowntimesDefaultsService.validateDowntimesInput(validate).subscribe((result: any): void => {
            this.cdr.markForCheck();
            if (!result.success) {
                this.errors = result.data.Downtime as GenericValidationError;
                this.isSending = false;
            }
            if (result.success) {
                const start = result.data.js_start;
                const end = result.data.js_end;
                this.items.forEach((element: ServiceDowntimeItem): void => {
                    element.start = start;
                    element.end = end;
                    element.comment = this.downtimeModal.comment;
                });


                this.percentage = 0;
                this.isSending = true;
                this.cdr.markForCheck();
                let count = this.items.length;
                let responseCount: number = 0
                let issueCount: number = 0;

                for (let i in this.items) {
                    const item = this.items[i];

                    this.ExternalCommandsService.schedule_maintenance(item).subscribe({
                        next: (value: any) => {
                            responseCount++
                            this.percentage = Math.round((responseCount / count) * 100);
                            this.cdr.markForCheck();

                            if (responseCount === count && issueCount === 0) {
                                // The timeout is not necessary. It is just to show the progress bar at 100% for a short time.
                                setTimeout(() => {
                                    // All records have been sent successfully. Reset the modal
                                    this.isSending = false;
                                    this.percentage = 0;
                                    this.hasErrors = false;

                                    this.hideModal();
                                }, 250);

                                const title: string = this.TranslocoService.translate('Downtimes added');
                                const msg: string = this.TranslocoService.translate('Commands added successfully to queue');

                                this.notyService.genericSuccess(msg, title);

                                setTimeout(() => {
                                    this.completed.emit(true);
                                }, 5000);
                            }

                        },
                        error: (error: HttpErrorResponse) => {
                            this.cdr.markForCheck();
                            issueCount++
                            this.hasErrors = true;

                            responseCount++;
                            this.percentage = Math.round((responseCount / count) * 100);

                            if (responseCount === count && issueCount > 0) {
                                // The timeout is not necessary. It is just to show the progress bar at 100% for a short time.
                                setTimeout(() => {
                                    this.isSending = false;
                                    this.percentage = 0;
                                }, 250);

                                this.notyService.genericError();

                                this.completed.emit(false);
                            }
                        }
                    });
                }
            }
        }));
    }


    ngOnInit() {
        this.subscriptions.add(this.modalService.modalState$.subscribe((state) => {
            if (state.id === 'serviceMaintenanceModal' && state.show === true) {
                this.getDefaults();
            }
        }));
        this.downtimeModal.comment = this.TranslocoService.translate('In progress');
    }

    ngOnDestroy() {
        this.subscriptions.unsubscribe();
    }

    getDefaults() {
        this.subscriptions.add(this.DowntimesDefaultsService.getDowntimesDefaultsConfiguration().subscribe(downtimeDefaults => {
            this.downtimeModal.from_date = this.createDateString(downtimeDefaults.js_from);
            this.downtimeModal.from_time = downtimeDefaults.from_time;
            this.downtimeModal.to_date = this.createDateString(downtimeDefaults.js_to);
            this.downtimeModal.to_time = downtimeDefaults.to_time;
            this.downtimeModal.comment = this.TranslocoService.translate(downtimeDefaults.comment);
            this.cdr.markForCheck();
        }));
    }


    createDateString = function (jsStringData: string) {
        let splitData = jsStringData.split(',');
        return splitData[0].trim() + '-' + splitData[1].trim() + '-' + splitData[2].trim()
    }

}
