import { ChangeDetectionStrategy, ChangeDetectorRef, Component, inject, OnDestroy, OnInit } from '@angular/core';
import {
    CardBodyComponent,
    CardComponent,
    CardFooterComponent,
    CardHeaderComponent,
    CardTitleDirective,
    ColComponent,
    ContainerComponent,
    FormCheckComponent,
    FormCheckInputDirective,
    FormCheckLabelDirective,
    FormControlDirective,
    FormDirective,
    InputGroupComponent,
    InputGroupTextDirective,
    NavComponent,
    NavItemComponent,
    RowComponent
} from '@coreui/angular';
import { TranslocoDirective, TranslocoPipe } from '@jsverse/transloco';
import { ActivatedRoute, Router, RouterLink } from '@angular/router';
import { PermissionDirective } from '../../../permissions/permission.directive';
import { FaIconComponent } from '@fortawesome/angular-fontawesome';
import { XsButtonDirective } from '../../../layouts/coreui/xsbutton-directive/xsbutton.directive';
import { Subscription } from 'rxjs';
import { ServicesService } from '../../services/services.service';
import { PermissionsService } from '../../../permissions/permissions.service';
import { TimezoneService } from '../../../services/timezone.service';
import { DebounceDirective } from '../../../directives/debounce.directive';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import {
    AutomapServicesstatus,
    getDefaultServicesAutomapParams,
    getServiceCurrentStateForApi,
    ServicesAutomapParams,
    ServicesAutomapRoot,
    ServicesCurrentStateFilter
} from '../../services/services.interface';
import { BlockLoaderComponent } from '../../../layouts/primeng/loading/block-loader/block-loader.component';
import { NgForOf, NgIf } from '@angular/common';
import { LocalStorageService } from '../../../services/local-storage.service';
import { ServicestatusIconAutomapComponent } from './servicestatus-icon-automap/servicestatus-icon-automap.component';
import { HoststatusIconAutomapComponent } from './hoststatus-icon-automap/hoststatus-icon-automap.component';
import { LabelLinkComponent } from '../../../layouts/coreui/label-link/label-link.component';
import {
    HostsBrowserModalComponent
} from '../../hosts/hosts-browser/hosts-browser-modal/hosts-browser-modal.component';
import { HostsBrowserModalService } from '../../hosts/hosts-browser/hosts-browser-modal/hosts-browser-modal.service';
import {
    ServiceBrowserModalComponent
} from '../../services/services-browser/service-browser-modal/service-browser-modal.component';
import {
    ServiceBrowserModalService
} from '../../services/services-browser/service-browser-modal/service-browser-modal.service';
import { NoRecordsComponent } from '../../../layouts/coreui/no-records/no-records.component';

@Component({
    selector: 'oitc-automaps-index',
    imports: [
        CardComponent,
        TranslocoDirective,
        RouterLink,
        PermissionDirective,
        FaIconComponent,
        CardTitleDirective,
        CardHeaderComponent,
        CardFooterComponent,
        CardBodyComponent,
        NavComponent,
        NavItemComponent,
        XsButtonDirective,
        FormCheckComponent,
        FormCheckInputDirective,
        FormCheckLabelDirective,
        InputGroupComponent,
        InputGroupTextDirective,
        ColComponent,
        ContainerComponent,
        DebounceDirective,
        FormControlDirective,
        FormDirective,
        FormsModule,
        ReactiveFormsModule,
        RowComponent,
        TranslocoPipe,
        BlockLoaderComponent,
        NgIf,
        ServicestatusIconAutomapComponent,
        HoststatusIconAutomapComponent,
        NgForOf,
        LabelLinkComponent,
        HostsBrowserModalComponent,
        ServiceBrowserModalComponent,
        NoRecordsComponent
    ],
    templateUrl: './automaps-index.component.html',
    styleUrl: './automaps-index.component.css',
    changeDetection: ChangeDetectionStrategy.OnPush
})
export class AutomapsIndexComponent implements OnInit, OnDestroy {

    public result?: ServicesAutomapRoot;

    public hideFilter: boolean = true;
    public params: ServicesAutomapParams = getDefaultServicesAutomapParams();

    public groupByHost: boolean = true;
    public showLabel: boolean = true;

    public currentStateFilter: ServicesCurrentStateFilter = {
        ok: false,
        warning: false,
        critical: false,
        unknown: false
    };

    private readonly ServicesService = inject(ServicesService);
    private subscriptions: Subscription = new Subscription();
    public readonly PermissionsService = inject(PermissionsService);
    private readonly TimezoneService = inject(TimezoneService);
    private readonly LocalStorageService = inject(LocalStorageService);
    public readonly route = inject(ActivatedRoute);
    public readonly router = inject(Router);
    private readonly HostsBrowserModalService = inject(HostsBrowserModalService);
    private readonly ServiceBrowserModalService = inject(ServiceBrowserModalService);
    private cdr = inject(ChangeDetectorRef);

    public ngOnInit() {
        // Restore values from local storage
        this.groupByHost = this.LocalStorageService.getItemWithDefault('automaps.groupByHost', 'true') === 'true';
        this.showLabel = this.LocalStorageService.getItemWithDefault('automaps.showLabel', 'true') === 'true';

        this.load();
    }

    public ngOnDestroy() {
        this.subscriptions.unsubscribe();
    }

    public updateLocalStorage() {
        this.LocalStorageService.setItem('automaps.groupByHost', (this.groupByHost ? 'true' : 'false'));
        this.LocalStorageService.setItem('automaps.showLabel', (this.showLabel ? 'true' : 'false'));
    }

    public load() {
        this.params['filter[Servicestatus.current_state][]'] = getServiceCurrentStateForApi(this.currentStateFilter);
        this.subscriptions.add(this.ServicesService.getServiceAutomap(this.params)
            .subscribe((result) => {
                this.result = result;
                this.cdr.markForCheck();
            })
        );
    }

    // Show or hide the filter
    // Called by (click) - no manual change detection required
    public toggleFilter() {
        this.hideFilter = !this.hideFilter;
    }

    public resetFilter() {
        this.params = getDefaultServicesAutomapParams();
        this.currentStateFilter = {
            ok: false,
            warning: false,
            critical: false,
            unknown: false
        };
        this.load();
    }

    // Callback when a filter has changed
    public onFilterChange(event: any) {
        this.load();
    }

    // Generic callback whenever a mass action (like delete all) has been finished
    public onMassActionComplete(success: boolean) {
        if (success) {
            this.load();
        }
    }

    public showHostStatusDetails(hostUuid: string) {
        this.PermissionsService.hasPermissionObservable(['hosts', 'browser']).subscribe(hasPermission => {

            if (hasPermission) {
                this.HostsBrowserModalService.openHostBrowserModal(hostUuid);
            }
        });
    }

    public showServiceStatusDetails(service: AutomapServicesstatus) {
        this.PermissionsService.hasPermissionObservable(['services', 'browser']).subscribe(hasPermission => {
            if (hasPermission) {
                this.ServiceBrowserModalService.openServiceBrowserModal(service.service_description);
            }
        });
    }

}
