import { ChangeDetectionStrategy, ChangeDetectorRef, Component, inject, OnDestroy, OnInit } from '@angular/core';
import {
    CardBodyComponent,
    CardComponent,
    CardHeaderComponent,
    CardTitleDirective,
    ColComponent,
    ProgressComponent,
    RowComponent
} from '@coreui/angular';
import { FaIconComponent } from '@fortawesome/angular-fontawesome';
import { TranslocoDirective } from '@jsverse/transloco';
import { RouterLink } from '@angular/router';
import { Subscription } from 'rxjs';
import { DashboardsService } from './dashboards.service';
import { DashboardsIndexRoot } from '../dashboards.interface';
import { AsyncPipe, NgIf } from '@angular/common';
import { PermissionDirective } from '../../../permissions/permission.directive';
import { PermissionsService } from '../../../permissions/permissions.service';


@Component({
    selector: 'oitc-dashboards-index',
    imports: [
        CardBodyComponent,
        AsyncPipe,
        CardComponent,
        CardHeaderComponent,
        CardTitleDirective,
        FaIconComponent,
        TranslocoDirective,
        RouterLink,
        RowComponent,
        ColComponent,
        ProgressComponent,
        NgIf,
        PermissionDirective
    ],
    templateUrl: './dashboards-index.component.html',
    styleUrl: './dashboards-index.component.scss',
    providers: [],
    changeDetection: ChangeDetectionStrategy.OnPush
})
export class DashboardsIndexComponent implements OnInit, OnDestroy {
    private readonly subscription: Subscription = new Subscription();
    private readonly DashboardsService: DashboardsService = inject(DashboardsService);
    private readonly cdr: ChangeDetectorRef = inject(ChangeDetectorRef);

    protected readonly PermissionsService = inject(PermissionsService);
    // Result
    protected dashboardsIndex?: DashboardsIndexRoot;

    // Hosts percentages
    protected hostsDownPercent: number = 0;
    protected hostsUpPercent: number = 0;
    protected hostsUnreachablePercent: number = 0;

    // Services percentages
    protected servicesOkPercent: number = 0;
    protected servicesWarningPercent: number = 0;
    protected servicesCriticalPercent: number = 0;
    protected servicesUnknownPercent: number = 0;

    // Unhandled Counts
    protected hostsUnhandledCount: number = 0;
    protected servicesUnhandledCount: number = 0;

    // Refresh interval
    protected refreshIntervalId: number = 0;

    ngOnInit(): void {
        this.getIndex();

        // Reload after 30 seconds.
        let interval: number = 30000;
        this.refreshIntervalId = setInterval(() => {
            this.getIndex();
        }, interval);
    }

    ngOnDestroy(): void {
        if (this.refreshIntervalId) {
            clearInterval(this.refreshIntervalId);
        }
        this.subscription.unsubscribe();
    }

    private getIndex(): void {
        this.subscription.add(this.DashboardsService.getIndex()
            .subscribe((result: DashboardsIndexRoot) => {
                this.dashboardsIndex = result;

                // Hosts percentages
                if (result.hostCount > 0) {
                    this.hostsUpPercent = Math.ceil(result.hostSummaryState.up / result.hostCount * 100);
                    this.hostsDownPercent = Math.ceil(result.hostSummaryState.down / result.hostCount * 100);
                    this.hostsUnreachablePercent = Math.ceil(result.hostSummaryState.unreachable / result.hostCount * 100);
                }

                // Services percentages
                if (result.serviceCount > 0) {
                    this.servicesOkPercent = Math.ceil(result.serviceSummaryState.ok / result.serviceCount * 100);
                    this.servicesWarningPercent = Math.ceil(result.serviceSummaryState.warning / result.serviceCount * 100);
                    this.servicesCriticalPercent = Math.ceil(result.serviceSummaryState.critical / result.serviceCount * 100);
                    this.servicesUnknownPercent = Math.ceil(result.serviceSummaryState.unknown / result.serviceCount * 100);
                }

                // Host unhandled count (nicht mit .reduce, weil ich Objekte angelegt habe, keine Arrays!)
                this.hostsUnhandledCount = 0;
                this.hostsUnhandledCount += this.dashboardsIndex.hostStatusCounts.not_handled[0];
                this.hostsUnhandledCount += this.dashboardsIndex.hostStatusCounts.not_handled[1];
                this.hostsUnhandledCount += this.dashboardsIndex.hostStatusCounts.not_handled[2];

                // Service unhandled count
                this.servicesUnhandledCount = 0;
                this.servicesUnhandledCount += this.dashboardsIndex.serviceStatusCounts.not_handled[0];
                this.servicesUnhandledCount += this.dashboardsIndex.serviceStatusCounts.not_handled[1];
                this.servicesUnhandledCount += this.dashboardsIndex.serviceStatusCounts.not_handled[2];
                this.servicesUnhandledCount += this.dashboardsIndex.serviceStatusCounts.not_handled[3];

                // cdr
                this.cdr.markForCheck();
            })
        );
    }

}

