import {
    ChangeDetectionStrategy,
    ChangeDetectorRef,
    Component,
    inject,
    Input,
    OnChanges,
    OnDestroy,
    OnInit,
    SimpleChanges
} from '@angular/core';
import { TranslocoDirective, TranslocoPipe, TranslocoService } from '@jsverse/transloco';
import {
    AlertComponent,
    CardBodyComponent,
    CardComponent,
    CardHeaderComponent,
    CardTitleDirective,
    ColComponent,
    ContainerComponent,
    DropdownComponent,
    DropdownItemDirective,
    DropdownMenuDirective,
    DropdownToggleDirective,
    FormCheckInputDirective,
    FormControlDirective,
    InputGroupComponent,
    InputGroupTextDirective,
    ModalService,
    NavComponent,
    NavItemComponent,
    RowComponent,
    TableDirective,
    TooltipDirective
} from '@coreui/angular';
import { Subscription } from 'rxjs';
import { ActivatedRoute, Router, RouterLink } from '@angular/router';
import { FaIconComponent } from '@fortawesome/angular-fontawesome';
import { AsyncPipe, NgForOf, NgIf } from '@angular/common';

import { HoststatusBrowser } from '../../hosts.interface';
import { SelectionServiceService } from '../../../../layouts/coreui/select-all/selection-service.service';
import { PermissionsService } from '../../../../permissions/permissions.service';
import { TimezoneConfiguration, TimezoneService } from '../../../../services/timezone.service';
import { NotyService } from '../../../../layouts/coreui/noty.service';
import {
    ExternalCommandsService,
    ServiceAcknowledgeItem,
    ServiceDowntimeItem,
    ServiceResetItem
} from '../../../../services/external-commands.service';
import { XsButtonDirective } from '../../../../layouts/coreui/xsbutton-directive/xsbutton.directive';
import { MatSort, MatSortHeader, Sort } from '@angular/material/sort';
import { FormsModule } from '@angular/forms';
import { DebounceDirective } from '../../../../directives/debounce.directive';
import { PermissionDirective } from '../../../../permissions/permission.directive';
import {
    ServicestatusIconComponent
} from '../../../../components/services/servicestatus-icon/servicestatus-icon.component';
import { ItemSelectComponent } from '../../../../layouts/coreui/select-all/item-select/item-select.component';
import {
    AcknowledgementIconComponent
} from '../../../acknowledgements/acknowledgement-icon/acknowledgement-icon.component';
import { DowntimeIconComponent } from '../../../downtimes/downtime-icon/downtime-icon.component';
import { PopoverGraphComponent } from '../../../../components/popover-graph/popover-graph.component';
import {
    getDefaultServicesIndexParams,
    getServiceCurrentStateForApi,
    ServicesCurrentStateFilter,
    ServicesIndexParams,
    ServicesIndexRoot
} from '../../../services/services.interface';
import { ServicesService } from '../../../services/services.service';
import { PaginatorChangeEvent } from '../../../../layouts/coreui/paginator/paginator.interface';
import { ExternalCommandsEnum } from '../../../../enums/external-commands.enum';
import { AcknowledgementTypes } from '../../../acknowledgements/acknowledgement-types.enum';
import { TableLoaderComponent } from '../../../../layouts/primeng/loading/table-loader/table-loader.component';
import { TrustAsHtmlPipe } from '../../../../pipes/trust-as-html.pipe';
import { NoRecordsComponent } from '../../../../layouts/coreui/no-records/no-records.component';
import { SelectAllComponent } from '../../../../layouts/coreui/select-all/select-all.component';
import {
    PaginateOrScrollComponent
} from '../../../../layouts/coreui/paginator/paginate-or-scroll/paginate-or-scroll.component';
import {
    ServiceMaintenanceModalComponent
} from '../../../../components/services/service-maintenance-modal/service-maintenance-modal.component';
import {
    ServiceAcknowledgeModalComponent
} from '../../../../components/services/service-acknowledge-modal/service-acknowledge-modal.component';

@Component({
    selector: 'oitc-hosts-browser-services-list',
    imports: [
        TranslocoDirective,
        CardBodyComponent,
        CardComponent,
        CardHeaderComponent,
        CardTitleDirective,
        FaIconComponent,
        NavComponent,
        NavItemComponent,
        PermissionDirective,
        AcknowledgementIconComponent,
        ColComponent,
        ContainerComponent,
        DowntimeIconComponent,
        DropdownComponent,
        DropdownItemDirective,
        DropdownMenuDirective,
        DropdownToggleDirective,
        ItemSelectComponent,
        MatSort,
        MatSortHeader,
        NgForOf,
        NgIf,
        NoRecordsComponent,
        PaginateOrScrollComponent,
        PopoverGraphComponent,
        RowComponent,
        SelectAllComponent,
        ServicestatusIconComponent,
        TableDirective,
        TableLoaderComponent,
        TranslocoPipe,
        XsButtonDirective,
        TooltipDirective,
        RouterLink,
        DebounceDirective,
        FormCheckInputDirective,
        FormControlDirective,
        InputGroupComponent,
        InputGroupTextDirective,
        FormsModule,
        AlertComponent,
        XsButtonDirective,
        MatSort,
        MatSortHeader,
        DebounceDirective,
        PermissionDirective,
        ServicestatusIconComponent,
        ItemSelectComponent,
        AcknowledgementIconComponent,
        DowntimeIconComponent,
        PopoverGraphComponent,
        TableLoaderComponent,
        AsyncPipe,
        TrustAsHtmlPipe,
        NoRecordsComponent,
        SelectAllComponent,
        PaginateOrScrollComponent,
        ServiceMaintenanceModalComponent,
        ServiceAcknowledgeModalComponent
    ],
    templateUrl: './hosts-browser-services-list.component.html',
    styleUrl: './hosts-browser-services-list.component.css',
    providers: [],
    changeDetection: ChangeDetectionStrategy.OnPush
})
export class HostsBrowserServicesListComponent implements OnInit, OnChanges, OnDestroy {

    @Input() hostUuid: string = '';
    @Input() hoststatus?: HoststatusBrowser;
    @Input() lastUpdated?: Date; // Change the date to trigger an update from an external component

    public activeTab: string = 'active';

    public timezone!: TimezoneConfiguration;
    public selectedItems: any[] = [];
    public userFullname: string = '';

    public params: ServicesIndexParams = getDefaultServicesIndexParams();

    public services?: ServicesIndexRoot;

    public currentStateFilter: ServicesCurrentStateFilter = {
        ok: false,
        warning: false,
        critical: false,
        unknown: false
    };

    private subscriptions: Subscription = new Subscription();
    private readonly ServicesService: ServicesService = inject(ServicesService);
    private readonly SelectionServiceService: SelectionServiceService = inject(SelectionServiceService);
    public readonly PermissionsService: PermissionsService = inject(PermissionsService);
    private readonly TimezoneService = inject(TimezoneService);
    public readonly router = inject(Router);
    private readonly notyService = inject(NotyService);
    private readonly TranslocoService = inject(TranslocoService);
    private readonly modalService = inject(ModalService);
    private readonly ExternalCommandsService = inject(ExternalCommandsService);

    private cdr = inject(ChangeDetectorRef);

    constructor(private route: ActivatedRoute) {
    }

    public ngOnInit(): void {
        this.getUserTimezone();
    }

    public ngOnChanges(changes: SimpleChanges): void {
        if (changes['hostUuid']) {
            this.load();
            return;
        }

        // Parent component wants to trigger an update
        if (changes['lastUpdated'] && !changes['lastUpdated'].isFirstChange()) {
            this.load();
            return;
        }

    }

    public ngOnDestroy(): void {
        this.subscriptions.unsubscribe();
    }

    private getUserTimezone() {
        this.subscriptions.add(this.TimezoneService.getTimezoneConfiguration().subscribe(data => {
            this.timezone = data;
            this.cdr.markForCheck();
        }));
    }

    public load() {
        this.loadActiveServices();
    };

    private loadActiveServices() {
        if (!this.hostUuid) {
            return;
        }

        // hostname is a UUID we add this filter to only get services for the current host
        this.params['filter[Servicestatus.hostname]'] = this.hostUuid;
        this.params['filter[Servicestatus.current_state][]'] = getServiceCurrentStateForApi(this.currentStateFilter);

        this.subscriptions.add(this.ServicesService.getIndex(this.params)
            .subscribe((services) => {
                this.services = services;
                this.userFullname = services.username;
                this.cdr.markForCheck();
            })
        );
    }


    // Callback when a filter has changed
    public onFilterChange(event: any) {
        this.params.page = 1;
        this.load();
    }

    // Callback when sort has changed
    public onSortChange(sort: Sort) {
        if (sort.direction) {
            this.params.sort = sort.active;
            this.params.direction = sort.direction;
            this.SelectionServiceService.deselectAll();
            this.load();
        }
    }

    // Callback for Paginator or Scroll Index Component
    public onPaginatorChange(change: PaginatorChangeEvent): void {
        this.params.page = change.page;
        this.params.scroll = change.scroll;
        this.SelectionServiceService.deselectAll();
        this.load();
    }

    public resetChecktime() {
        const commands = this.SelectionServiceService.getSelectedItems().map((item): ServiceResetItem => {
            return {
                command: ExternalCommandsEnum.rescheduleService,
                hostUuid: item.hostname,
                serviceUuid: item.service_description
            };

        });
        if (commands.length === 0) {
            const message = this.TranslocoService.translate('No items selected!');
            this.notyService.genericError(message);
            return;
        }

        commands.forEach(command => {
            this.subscriptions.add(this.ExternalCommandsService.reset_check_time(command).subscribe((result) => {
            }));
        });

        const title = this.TranslocoService.translate('Reschedule');
        const msg: string = this.TranslocoService.translate('Commands added successfully to queue');

        this.notyService.genericSuccess(msg, title);

        setTimeout(() => {
            this.onMassActionComplete(true);
        }, 5000);

    }

    public toggleDowntimeModal() {
        let items: ServiceDowntimeItem[] = [];
        items = this.SelectionServiceService.getSelectedItems().map((item): ServiceDowntimeItem => {
            return {
                command: ExternalCommandsEnum.submitServiceDowntime,
                hostUuid: item.hostname,
                serviceUuid: item.service_description,
                start: 0,
                end: 0,
                author: this.userFullname,
                comment: '',
            };
        });

        this.selectedItems = items;
        if (items.length === 0) {
            const message = this.TranslocoService.translate('No items selected!');
            this.notyService.genericError(message);
            return;
        }
        this.modalService.toggle({
            show: true,
            id: 'serviceMaintenanceModal',
        });
    }

    public acknowledgeStatus() {
        let items: ServiceAcknowledgeItem[] = [];
        items = this.SelectionServiceService.getSelectedItems().map((item): ServiceAcknowledgeItem => {
            return {
                command: ExternalCommandsEnum.submitServicestateAck,
                hostUuid: item.hostname,
                serviceUuid: item.service_description,
                sticky: false,
                notify: true,
                author: this.userFullname,
                comment: '',
            };
        });
        this.selectedItems = items;
        if (items.length === 0) {
            const message = this.TranslocoService.translate('No items selected!');
            this.notyService.genericError(message);
            return;
        }
        this.modalService.toggle({
            show: true,
            id: 'serviceAcknowledgeModal',
        });
    }

    public onMassActionComplete(success: boolean) {
        if (success) {
            this.load();
        }
    }

    protected readonly AcknowledgementTypes = AcknowledgementTypes;
    protected readonly Number = Number;
    protected readonly String = String;
}
