import {
    ChangeDetectionStrategy,
    ChangeDetectorRef,
    Component,
    inject,
    OnDestroy,
    OnInit,
    Pipe,
    PipeTransform
} from '@angular/core';
import { Subscription } from 'rxjs';
import { UsergroupsService } from '../usergroups.service';
import { NotyService } from '../../../layouts/coreui/noty.service';
import { ActivatedRoute, Router, RouterLink } from '@angular/router';
import { HistoryService } from '../../../history.service';
import { TranslocoDirective, TranslocoPipe, TranslocoService } from '@jsverse/transloco';
import { PermissionsService } from '../../../permissions/permissions.service';
import { GenericIdResponse, GenericResponseWrapper, GenericValidationError } from '../../../generic-responses';
import { Acos, UsergroupDefault } from '../usergroups.interface';
import { KeyValuePipe, NgClass, NgForOf, NgIf } from '@angular/common';
import { FaIconComponent } from '@fortawesome/angular-fontawesome';
import { PermissionDirective } from '../../../permissions/permission.directive';
import { FormLoaderComponent } from '../../../layouts/primeng/loading/form-loader/form-loader.component';
import { FormsModule } from '@angular/forms';
import {
    AlertComponent,
    CardBodyComponent,
    CardComponent,
    CardFooterComponent,
    CardHeaderComponent,
    CardTitleDirective,
    ColComponent,
    DropdownComponent,
    DropdownDividerDirective,
    DropdownItemDirective,
    DropdownMenuDirective,
    DropdownToggleDirective,
    FormCheckComponent,
    FormCheckInputDirective,
    FormCheckLabelDirective,
    FormControlDirective,
    FormLabelDirective,
    InputGroupComponent,
    InputGroupTextDirective,
    NavComponent,
    NavItemComponent,
    RowComponent
} from '@coreui/angular';
import { BackButtonDirective } from '../../../directives/back-button.directive';
import { XsButtonDirective } from '../../../layouts/coreui/xsbutton-directive/xsbutton.directive';
import { RequiredIconComponent } from '../../../components/required-icon/required-icon.component';
import { FormErrorDirective } from '../../../layouts/coreui/form-error.directive';
import { FormFeedbackComponent } from '../../../layouts/coreui/form-feedback/form-feedback.component';
import { TrueFalseDirective } from '../../../directives/true-false.directive';

@Component({
    selector: 'oitc-usergroups-edit',
    imports: [
        NgIf,
        TranslocoDirective,
        RouterLink,
        FaIconComponent,
        PermissionDirective,
        FormLoaderComponent,
        FormsModule,
        CardComponent,
        CardHeaderComponent,
        NavComponent,
        NavItemComponent,
        BackButtonDirective,
        XsButtonDirective,
        CardBodyComponent,
        FormLabelDirective,
        RequiredIconComponent,
        FormControlDirective,
        FormErrorDirective,
        FormFeedbackComponent,
        AlertComponent,
        RowComponent,
        ColComponent,
        InputGroupComponent,
        InputGroupTextDirective,
        TranslocoPipe,
        DropdownComponent,
        DropdownToggleDirective,
        DropdownMenuDirective,
        DropdownItemDirective,
        DropdownDividerDirective,
        FormCheckComponent,
        FormCheckInputDirective,
        FormCheckLabelDirective,
        KeyValuePipe,
        NgForOf,
        TrueFalseDirective,
        CardFooterComponent,
        NgClass,
        CardTitleDirective
    ],
    templateUrl: './usergroups-edit.component.html',
    styleUrl: './usergroups-edit.component.css',
    changeDetection: ChangeDetectionStrategy.OnPush
})
export class UsergroupsEditComponent implements OnInit, OnDestroy {
    private readonly subscriptions: Subscription = new Subscription();
    private readonly UsergroupsService: UsergroupsService = inject(UsergroupsService);
    private readonly notyService: NotyService = inject(NotyService);
    private readonly router: Router = inject(Router);
    private readonly route: ActivatedRoute = inject(ActivatedRoute);
    private readonly HistoryService: HistoryService = inject(HistoryService);
    private readonly TranslocoService = inject(TranslocoService);
    private readonly PermissionsService = inject(PermissionsService);
    private readonly cdr = inject(ChangeDetectorRef);

    protected systemname: string = 'openITCOCKPIT';
    protected errors: GenericValidationError | null = null;
    protected acos: Acos = {acos: {}} as Acos;
    protected controllerFilter: string = '';
    protected readonly keepOrder = keepOrder;
    protected post!: UsergroupDefault;

    public ngOnInit(): void {
        this.route.queryParams.subscribe(params => {
            const id = Number(this.route.snapshot.paramMap.get('id'));
            this.loadAcos();
            this.loadUsergroup(id);
        });
    }

    private loadUsergroup(id: number) {
        this.subscriptions.add(this.UsergroupsService.getEdit(id).subscribe(
            (result) => {
                this.post = {
                    Usergroup: result.usergroup,
                    Acos: result.acos
                };
                //Set permissions of current user group to $scope.post.Acos;
                if ((result.usergroup.aro) && result.usergroup.aro.acos) {
                    for (let usergroupAco in result.usergroup.aro.acos) {
                        let usergroupAcoId = result.usergroup.aro.acos[parseInt(usergroupAco)].id;
                        //Deny all by default
                        this.post.Acos[usergroupAcoId] = 0;
                        if (result.usergroup.aro.acos[parseInt(usergroupAco)]._joinData._create === "1") {
                            //Only enable what is enabled in the database
                            this.post.Acos[usergroupAcoId] = 1;
                        }
                    }
                }
                this.cdr.markForCheck();
            }));
    }

    private loadAcos(): void {
        this.subscriptions.add(this.UsergroupsService.loadAcos().subscribe((acos: Acos) => {
            this.acos = acos;
            this.cdr.markForCheck();
        }));
    }

    public ngOnDestroy(): void {
        this.subscriptions.unsubscribe();
    }

    protected editUserrole() {
        this.subscriptions.add(this.UsergroupsService.updateUsergroup(this.post)
            .subscribe((result: GenericResponseWrapper) => {
                this.cdr.markForCheck();
                if (result.success) {
                    const response: GenericIdResponse = result.data as GenericIdResponse;

                    const title: string = this.TranslocoService.translate('User role');
                    const msg: string = this.TranslocoService.translate('updated successfully');
                    const url: (string | number)[] = ['usergroups', 'edit', response.id];

                    // Reload user permissions just in the current logged in is part of the user group
                    this.PermissionsService.loadPermissions();
                    this.notyService.genericSuccess(msg, title, url);
                    this.HistoryService.navigateWithFallback(['/usergroups/index']);
                    return;
                }

                // Error
                this.notyService.genericError();
                const errorResponse: GenericValidationError = result.data as GenericValidationError;
                if (result) {
                    this.errors = errorResponse;
                }
            })
        );
    }

    protected forceTicks(actionToTick: string, tick: boolean) {
        for (let aco in this.acos.acos) {
            for (let controller in this.acos.acos[aco].children) {
                let isModule = this.acos.acos[aco].children[controller].alias.endsWith('Module');

                if (isModule) {
                    for (let pluginController in this.acos.acos[aco].children[controller].children) {
                        for (let action in this.acos.acos[aco].children[controller].children[pluginController].children) {
                            let actionName = this.acos.acos[aco].children[controller].children[pluginController].children[action].alias;
                            let acoId = this.acos.acos[aco].children[controller].children[pluginController].children[action].id;
                            if (actionName === actionToTick || actionToTick === 'all') {
                                this.post.Acos[acoId] = tick ? 1 : 0;
                            }
                        }
                    }
                    continue;
                }
                for (let action in this.acos.acos[aco].children[controller].children) {
                    let acoId = this.acos.acos[aco].children[controller].children[action].id;
                    let actionName = this.acos.acos[aco].children[controller].children[action].alias;
                    if (actionName === actionToTick || actionToTick === 'all') {
                        this.post.Acos[acoId] = tick ? 1 : 0;
                    }
                }
            }
        }
        this.cdr.markForCheck();
    }

    protected readonly Object = Object;
}

const keepOrder = (a: any, b: any) => a;

// This pipe uses the angular keyvalue pipe. but doesn't change order.
@Pipe({
    standalone: true,
    name: 'defaultOrderKeyvalue'
})
export class DefaultOrderKeyValuePipe extends KeyValuePipe implements PipeTransform {
    override transform(value: any, ...args: any[]): any {
        return super.transform(value, keepOrder);
    }
}
