import { ChangeDetectionStrategy, ChangeDetectorRef, Component, inject, OnDestroy, OnInit } from '@angular/core';
import { UserPost, UserTimezonesSelect } from '../users.interface';
import { GenericIdResponse, GenericValidationError } from '../../../generic-responses';
import { SelectKeyValue, SelectKeyValueString } from '../../../layouts/primeng/select.interface';
import { PermissionsService } from '../../../permissions/permissions.service';
import { Subscription } from 'rxjs';
import { UsersService } from '../users.service';
import { NotyService } from '../../../layouts/coreui/noty.service';
import { HistoryService } from '../../../history.service';
import { TranslocoDirective, TranslocoService } from '@jsverse/transloco';
import { RouterLink } from '@angular/router';
import { FaIconComponent } from '@fortawesome/angular-fontawesome';
import { PermissionDirective } from '../../../permissions/permission.directive';
import {
    CardBodyComponent,
    CardComponent,
    CardFooterComponent,
    CardHeaderComponent,
    CardTitleDirective,
    FormCheckComponent,
    FormCheckInputDirective,
    FormCheckLabelDirective,
    FormControlDirective,
    FormDirective,
    FormLabelDirective,
    NavComponent,
    NavItemComponent
} from '@coreui/angular';
import { FormsModule } from '@angular/forms';
import { BackButtonDirective } from '../../../directives/back-button.directive';
import { XsButtonDirective } from '../../../layouts/coreui/xsbutton-directive/xsbutton.directive';
import { RequiredIconComponent } from '../../../components/required-icon/required-icon.component';
import { SelectComponent } from '../../../layouts/primeng/select/select/select.component';
import { FormErrorDirective } from '../../../layouts/coreui/form-error.directive';
import { FormFeedbackComponent } from '../../../layouts/coreui/form-feedback/form-feedback.component';
import { TrueFalseDirective } from '../../../directives/true-false.directive';
import { NgOptionTemplateDirective, NgSelectComponent } from '@ng-select/ng-select';
import { NgOptionHighlightDirective } from '@ng-select/ng-option-highlight';
import { UsergroupsService } from '../../usergroups/usergroups.service';

@Component({
    selector: 'oitc-users-add',
    imports: [
        TranslocoDirective,
        RouterLink,
        FaIconComponent,
        PermissionDirective,
        FormDirective,
        FormsModule,
        CardComponent,
        CardHeaderComponent,
        NavComponent,
        CardTitleDirective,
        NavItemComponent,
        BackButtonDirective,
        CardBodyComponent,
        XsButtonDirective,
        RequiredIconComponent,
        SelectComponent,
        FormErrorDirective,
        FormFeedbackComponent,
        FormLabelDirective,
        FormControlDirective,
        FormCheckComponent,
        FormCheckInputDirective,
        TrueFalseDirective,
        FormCheckLabelDirective,
        NgSelectComponent,
        NgOptionTemplateDirective,
        NgOptionHighlightDirective,
        CardFooterComponent
    ],
    templateUrl: './users-add.component.html',
    styleUrl: './users-add.component.css',
    changeDetection: ChangeDetectionStrategy.OnPush
})
export class UsersAddComponent implements OnInit, OnDestroy {
    public createAnother: boolean = false;
    public post: UserPost = this.getDefaultPost();
    public errors: GenericValidationError | null = null;

    public usergroups: SelectKeyValue[] = [];
    public dateformats: SelectKeyValueString[] = [];
    public timezones: UserTimezonesSelect[] = [];
    public serverTime: string = '';
    public serverTimeZone: string = '';

    public readonly PermissionsService: PermissionsService = inject(PermissionsService);

    private subscriptions: Subscription = new Subscription();
    private readonly UsersService: UsersService = inject(UsersService);
    private readonly UsergroupsService: UsergroupsService = inject(UsergroupsService);
    private readonly TranslocoService: TranslocoService = inject(TranslocoService);
    private readonly notyService = inject(NotyService);
    private readonly HistoryService: HistoryService = inject(HistoryService);
    private cdr = inject(ChangeDetectorRef);


    public ngOnInit(): void {
        this.loadUsergroups();
        this.loadDateformats();
    }

    public ngOnDestroy(): void {
        this.subscriptions.unsubscribe();
    }

    private getDefaultPost(): UserPost {
        return {
            usergroup_id: 0,
            username: '',
            firstname: '',
            lastname: '',
            showstatsinmenu: 0,
            paginatorlength: 25,
            dateformat: 'H:i:s - d.m.Y',
            timezone: 'Europe/Berlin',
            password: '',
            confirm_password: ''
        }
    }

    public loadUsergroups() {
        this.subscriptions.add(this.UsergroupsService.loadUsergroups().subscribe((result) => {
            this.usergroups = result;
            this.cdr.markForCheck();
        }));
    }

    public loadDateformats() {
        this.subscriptions.add(this.UsersService.getDateformats().subscribe((result) => {
            this.dateformats = result.dateformats;
            this.post.dateformat = result.defaultDateFormat;
            this.timezones = result.timezones;
            this.serverTimeZone = result.serverTimeZone;
            this.serverTime = result.serverTime;
            this.cdr.markForCheck();
        }));
    }

    public submit() {
        this.subscriptions.add(this.UsersService.add(this.post)
            .subscribe((result) => {
                this.cdr.markForCheck();
                if (result.success) {
                    const response = result.data as GenericIdResponse;
                    const title = this.TranslocoService.translate('User');
                    const msg = this.TranslocoService.translate('created successfully');
                    const url = ['users', 'edit', response.id];

                    this.notyService.genericSuccess(msg, title, url);

                    if (!this.createAnother) {
                        this.HistoryService.navigateWithFallback(['/users/index']);
                        return;
                    }
                    this.post = this.getDefaultPost();
                    this.notyService.scrollContentDivToTop();
                    this.errors = null;
                    return;
                }

                // Error
                const errorResponse = result.data as GenericValidationError;
                this.notyService.genericError();
                if (result) {
                    this.errors = errorResponse;
                }
            }));
    }
}
