import { inject, Injectable } from '@angular/core';
import { catchError, map, Observable, of } from 'rxjs';
import { HttpClient } from '@angular/common/http';
import { PROXY_PATH } from '../../tokens/proxy-path.token';
import {
    LoginGetRoot,
    UserDateformatsRoot,
    UserPost,
    UsersEditResponse,
    UsersIndexParams,
    UsersIndexRoot,
    UserTimezoneGroup,
    UserTimezonesSelect
} from './users.interface';
import { DeleteAllItem } from '../../layouts/coreui/delete-all-modal/delete-all.interface';
import { SelectKeyValueString } from '../../layouts/primeng/select.interface';
import { GenericIdResponse, GenericResponseWrapper, GenericValidationError } from '../../generic-responses';


@Injectable({
    providedIn: 'root'
})
export class UsersService {

    constructor() {
    }

    private readonly http: HttpClient = inject(HttpClient);
    private readonly proxyPath: string = inject(PROXY_PATH);

    public getLoginDetails(): Observable<LoginGetRoot> {
        const proxyPath = this.proxyPath;
        return this.http.get<LoginGetRoot>(`${proxyPath}/users/login.json`).pipe(
            map(data => {
                return data;
            })
        );
    }

    // Generic function for the Delete All Modal
    public delete(item: DeleteAllItem): Observable<Object> {
        const proxyPath = this.proxyPath;
        return this.http.post(`${proxyPath}/users/delete/${item.id}.json?angular=true`, {});
    }

    public getDateformats(): Observable<UserDateformatsRoot> {
        const proxyPath = this.proxyPath;
        return this.http.get<{
            dateformats: SelectKeyValueString[],
            defaultDateFormat: string,
            timezones: UserTimezoneGroup,
            serverTime: string,
            serverTimeZone: string
        }>(`${proxyPath}/users/loadDateformats.json`, {
            params: {
                angular: true
            }
        }).pipe(
            map(data => {
                // We reformat the timezones, so we can use them in a select box

                let timezones: UserTimezonesSelect[] = [];
                for (var timezoneGroup in data.timezones) {
                    for (var timezone in data.timezones[timezoneGroup]) {
                        timezones.push({
                            group: timezoneGroup,
                            value: timezone,
                            name: data.timezones[timezoneGroup][timezone]
                        });
                    }

                }

                let result: UserDateformatsRoot = {
                    dateformats: data.dateformats,
                    defaultDateFormat: data.defaultDateFormat,
                    timezones: timezones,
                    serverTime: data.serverTime,
                    serverTimeZone: data.serverTimeZone
                };
                return result;
            })
        );
    }

    public getIndex(params: UsersIndexParams): Observable<UsersIndexRoot> {
        const proxyPath = this.proxyPath;
        return this.http.get<UsersIndexRoot>(`${proxyPath}/Users/index.json`, {
            params: params as {} // cast UsersIndexParams into object
        }).pipe(
            map(data => {
                return data;
            })
        )
    }

    public add(user: UserPost): Observable<GenericResponseWrapper> {
        const proxyPath = this.proxyPath;
        return this.http.post<any>(`${proxyPath}/users/add.json?angular=true`, {
            User: user
        })
            .pipe(
                map(data => {
                    // Return true on 200 Ok
                    return {
                        success: true,
                        data: data.user as GenericIdResponse
                    };
                }),
                catchError((error: any) => {
                    const err = error.error.error as GenericValidationError;
                    return of({
                        success: false,
                        data: err
                    });
                })
            );
    }

    public getUserEdit(id: number): Observable<UsersEditResponse> {
        const proxyPath = this.proxyPath;
        return this.http.get<UsersEditResponse>(`${proxyPath}/users/edit/${id}.json`, {
            params: {
                angular: true
            }
        }).pipe(
            map(data => {
                return data;
            })
        );
    }

    public saveUserEdit(user: UserPost): Observable<GenericResponseWrapper> {
        const proxyPath = this.proxyPath;
        return this.http.post<any>(`${proxyPath}/users/edit/${user.id}.json?angular=true`, {
            User: user
        })
            .pipe(
                map(data => {
                    // Return true on 200 Ok
                    return {
                        success: true,
                        data: data.user as GenericIdResponse
                    };
                }),
                catchError((error: any) => {
                    const err = error.error.error as GenericValidationError;
                    return of({
                        success: false,
                        data: err
                    });
                })
            );
    }
}
